/* rnaparam.c
 * rna parameters at order 2 in IS,
 * mostly zuker style + pseudoknots (P11, P12, P13, wkn)
 *
 * 
 */
          
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cfg.h"
#include "proto.h"
#include "squid.h"

/* Function: Parameters2_Zkn()
 * 
 * Purpose:  Input of thermodynamic RNA folding parameters.
 *           with Zuker sign convention, adding knots
 *
 *           ranpar  - RETURN: structure containing data.   
 *           
 * Return:   (void)
 *           rnapar is filled.          
 */

void
Parameters2_Zkn(struct rnapar_2 *rnapar)
{
  int i,j;
  int k,l;

  int **iseq;
  int tlp, ntlp, isc;
  
  /* Freier/Turner/Tinoco Rules
   * Thermodynamic parameters for RNA structure
   */

  /* stacking energies */
  float stack03[4][4] = {{+0.4, +0.4, +0.4, -0.9},
			 {+0.4, +0.4, -2.1, +0.4},
			 {+0.4, -1.7, +0.4, -0.5},
			 {-0.9, +0.4, -1.0, +0.4}};

  float stack30[4][4] = {{+0.4, +0.4, +0.4, -1.1},
			 {+0.4, +0.4, -2.3, +0.4},
			 {+0.4, -1.8, +0.4, -0.8},
			 {-0.9, +0.4, -1.1, +0.4}};

  float stack12[4][4] = {{+0.4, +0.4, +0.4, -1.8},
			 {+0.4, +0.4, -2.9, +0.4},
			 {+0.4, -2.0, +0.4, -1.2},
			 {-1.7, +0.4, -1.9, +0.4}};

  float stack21[4][4] = {{+0.4, +0.4, +0.4, -2.3},
			 {+0.4, +0.4, -3.4, +0.4},
			 {+0.4, -2.9, +0.4, -1.4},
			 {-2.1, +0.4, -2.1, +0.4}};

  float stack23[4][4] = {{+0.4, +0.4, +0.4, -1.1},
			 {+0.4, +0.4, -2.1, +0.4},
			 {+0.4, -1.9, +0.4, -0.4},
			 {-1.0, +0.4, +1.5, +0.4}};

  float stack32[4][4] = {{+0.4, +0.4, +0.4, -0.8},
			 {+0.4, +0.4, -1.4, +0.4},
			 {+0.4, -1.2, +0.4, -0.2},
			 {-0.5, +0.4, -0.4, +0.4}};

  /* hairpin terminal energies */

  float tstckh03[4][4] = {{-0.8, -1.0, -1.7, -1.0},
			  {-0.7, -0.7, -0.7, -0.7},
			  {-1.5, -1.0, -1.0, -1.0},
			  {-0.8, -0.8, -0.8, -0.8}};

  float tstckh30[4][4] = {{-1.0, -0.8, -1.8, -0.9},
			  {-0.7, -0.6, -0.3, -0.5},
			  {-1.8, -0.9, -1.2, -0.9},
			  {-0.3, -0.6, -0.3, -0.5}};

  float tstckh12[4][4] = {{-1.4, -2.0, -2.1, -1.9},
			  {-1.0, -1.1, -1.0, -0.8},
			  {-2.1, -1.9, -1.4, -1.9},
			  {-1.4, -1.5, -1.4, -1.2}};

  float tstckh21[4][4] = {{-1.1, -1.3, -2.0, -1.3},
			  {-1.1, -0.6, -0.6, -0.5},
			  {-2.3, -1.5, -1.4, -1.5},
			  {-0.8, -0.8, -0.8, -0.7}};

  float tstckh23[4][4] = {{-0.8, -1.0, -1.7, -1.0},
			  {-0.7, -0.7, -0.7, -0.7},
			  {-1.5, -1.0, -1.0, -1.0},
			  {-0.8, -0.8, -0.8, -0.8}};

  float tstckh32[4][4] = {{-1.2, -1.4, -2.0, -1.4},
			  {-0.9, -0.9, -0.7, -0.7},
			  {-2.0, -1.4, -1.3, -1.4},
			  {-0.9, -1.1, -0.9, -0.9}};


  /* interior loop terminal energies */

  float tstcki03[4][4] = {{-1.0, -1.0, -2.2, -0.5},
			  {-1.0, -1.0, -0.2, -1.0},
			  {-2.2, -0.5, -1.0, -0.5},
			  {-0.3, -1.0, -0.3, -2.0}};
  
  float tstcki30[4][4] = {{-1.0, -1.0, -2.2, -0.4},
			  {-1.0, -1.0, +0.2, -1.0},
			  {-2.2, -0.4, -1.0, -0.4},
			  {+0.2, -1.0, +0.2, -2.0}};
  
  float tstcki12[4][4] = {{-1.5, -1.5, -2.7, -1.9},
			  {-1.5, -1.5, -1.0, -1.5},
			  {-2.7, -1.9, -1.5, -1.9},
			  {-1.4, -1.5, -1.4, -2.5}};
  
  float tstcki21[4][4] = {{-1.5, -1.5, -2.7, -1.3},
			  {-1.5, -1.5, -0.6, -1.5},
			  {-2.7, -1.5, -1.5, -1.5},
			  {-0.8, -1.5, -0.8, -2.5}};
  
  float tstcki23[4][4] = {{-1.5, -1.5, -2.7, -1.3},
			  {-1.5, -1.5, -0.6, -1.5},
			  {-2.7, -1.5, -1.5, -1.5},
			  {-0.8, -1.5, -0.8, -2.5}};
  
  float tstcki32[4][4] = {{-1.0, -1.0, -2.2, -0.4},
			  {-1.0, -1.0, +0.2, -1.0},
			  {-2.2, -0.4, -1.0, -0.4},
			  {+0.2, -1.0, +0.2, -2.0}};
  
  /* dangling energies */
  
  float dangle50[4][4] =  {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.3},
			   {BIGFLOAT, BIGFLOAT,     -0.5, BIGFLOAT},
			   {BIGFLOAT,    -0.2, BIGFLOAT,      -0.2},
			   {    -0.3, BIGFLOAT,    -0.2, BIGFLOAT}};
  
  float dangle51[4][4] = {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.3},
			  {BIGFLOAT, BIGFLOAT,     -0.3, BIGFLOAT},
			  {BIGFLOAT,     -0.3, BIGFLOAT,     -0.2},
			  {    -0.1, BIGFLOAT,     -0.2, BIGFLOAT}};
  
  float dangle52[4][4] = {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.4},
			  {BIGFLOAT, BIGFLOAT,     -0.2, BIGFLOAT},
			  {BIGFLOAT,     +0.0, BIGFLOAT,     -0.2},
			  {    -0.2, BIGFLOAT,     -0.2, BIGFLOAT}};
  
  float dangle53[4][4] =  {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.2},
			   {BIGFLOAT, BIGFLOAT,     -0.1, BIGFLOAT},
			   {BIGFLOAT,     +0.0, BIGFLOAT,     -0.2},
			   {    -0.2, BIGFLOAT,     -0.2, BIGFLOAT}};
  
  
  float dangle30[4][4] =  {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.7},
			   {BIGFLOAT, BIGFLOAT,     -1.1, BIGFLOAT},
			   {BIGFLOAT,     -1.7, BIGFLOAT,     -1.2},
			   {    -0.8, BIGFLOAT,     -0.8, BIGFLOAT}};
  
  float dangle31[4][4] = {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.1},
			  {BIGFLOAT, BIGFLOAT,     -0.4, BIGFLOAT},
			  {BIGFLOAT,     -0.8, BIGFLOAT,     -0.5},
			  {    -0.5, BIGFLOAT,     -0.5, BIGFLOAT}};
  
  float dangle32[4][4] = {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.7},
			  {BIGFLOAT, BIGFLOAT,     -1.3, BIGFLOAT},
			  {BIGFLOAT,     -1.7, BIGFLOAT,     -1.2},
			  {    -0.8, BIGFLOAT,     -0.8, BIGFLOAT}};
  
  float dangle33[4][4] = {{BIGFLOAT, BIGFLOAT, BIGFLOAT,     -0.1},
			  {BIGFLOAT, BIGFLOAT,     -0.6, BIGFLOAT},
			  {BIGFLOAT,     -1.2, BIGFLOAT,     -0.7},
			  {    -0.6, BIGFLOAT,     -0.6, BIGFLOAT}};
  
  /* give the sequence of the ultrastable tetraloops */
  char tloop[13][4] = {"GAAA",
		       "GCAA",
		       "GAGA",
		       "GUGA",
		       "GGAA",
		       "UUCG",
		       "UACG",
		       "GCGA",
		       "UCCG",
		       "GUAA",
		       "CUUG",
		       "AUUU",
		       "UUUA"};
  /* loop energies */
  
  rnapar->inter[0]  = BIGFLOAT;
  rnapar->inter[1]  = BIGFLOAT;
  rnapar->inter[2]  = 4.1;
  rnapar->inter[3]  = 5.1;
  rnapar->inter[4]  = 4.9;
  rnapar->inter[5]  = 5.3;
  rnapar->inter[6]  = 5.7;
  rnapar->inter[7]  = 5.9;
  rnapar->inter[8]  = 6.0;
  rnapar->inter[9]  = 6.1;
  rnapar->inter[10] = 6.3;
  rnapar->inter[11] = 6.4;
  rnapar->inter[12] = 6.4;
  rnapar->inter[13] = 6.5;
  rnapar->inter[14] = 6.6;
  rnapar->inter[15] = 6.7;
  rnapar->inter[16] = 6.8;
  rnapar->inter[17] = 6.8;
  rnapar->inter[18] = 6.9;
  rnapar->inter[19] = 6.9;
  rnapar->inter[20] = 7.0;
  rnapar->inter[21] = 7.1;
  rnapar->inter[22] = 7.1;
  rnapar->inter[23] = 7.1;
  rnapar->inter[24] = 7.2;
  rnapar->inter[25] = 7.2;
  rnapar->inter[26] = 7.3;
  rnapar->inter[27] = 7.3;
  rnapar->inter[28] = 7.4;
  rnapar->inter[29] = 7.4;
  rnapar->inter[30] = 7.4;

  rnapar->bulge[0]  = BIGFLOAT;
  rnapar->bulge[1]  = 3.9;
  rnapar->bulge[2]  = 3.1;
  rnapar->bulge[3]  = 3.5;
  rnapar->bulge[4]  = 4.2;
  rnapar->bulge[5]  = 4.8;
  rnapar->bulge[6]  = 5.0;
  rnapar->bulge[7]  = 5.2;
  rnapar->bulge[8]  = 5.3;
  rnapar->bulge[9]  = 5.4;
  rnapar->bulge[10] = 5.5;
  rnapar->bulge[11] = 5.7;
  rnapar->bulge[12] = 5.7;
  rnapar->bulge[13] = 5.8;
  rnapar->bulge[14] = 5.9;
  rnapar->bulge[15] = 6.0;
  rnapar->bulge[16] = 6.1;
  rnapar->bulge[17] = 6.1;
  rnapar->bulge[18] = 6.2;
  rnapar->bulge[19] = 6.2;
  rnapar->bulge[20] = 6.3;
  rnapar->bulge[21] = 6.3;
  rnapar->bulge[22] = 6.4;
  rnapar->bulge[23] = 6.4;
  rnapar->bulge[24] = 6.5;
  rnapar->bulge[25] = 6.5;
  rnapar->bulge[26] = 6.5;
  rnapar->bulge[27] = 6.6;
  rnapar->bulge[28] = 6.7;
  rnapar->bulge[29] = 6.7;
  rnapar->bulge[30] = 6.7;

  rnapar->hairpin[0]  = BIGFLOAT;
  rnapar->hairpin[1]  = BIGFLOAT;
  rnapar->hairpin[2]  = BIGFLOAT;
  rnapar->hairpin[3]  = 4.1;
  rnapar->hairpin[4]  = 4.9;
  rnapar->hairpin[5]  = 4.4;
  rnapar->hairpin[6]  = 4.7;
  rnapar->hairpin[7]  = 5.0;
  rnapar->hairpin[8]  = 5.1;
  rnapar->hairpin[9]  = 5.2;
  rnapar->hairpin[10] = 5.3;
  rnapar->hairpin[11] = 5.4;
  rnapar->hairpin[12] = 5.5;
  rnapar->hairpin[13] = 5.6;
  rnapar->hairpin[14] = 5.7;
  rnapar->hairpin[15] = 5.8;
  rnapar->hairpin[16] = 5.8;
  rnapar->hairpin[17] = 5.9;
  rnapar->hairpin[18] = 5.9;
  rnapar->hairpin[19] = 6.0;
  rnapar->hairpin[20] = 6.1;
  rnapar->hairpin[21] = 6.1;
  rnapar->hairpin[22] = 6.2;
  rnapar->hairpin[23] = 6.2;
  rnapar->hairpin[24] = 6.3;
  rnapar->hairpin[25] = 6.3;
  rnapar->hairpin[26] = 6.3;
  rnapar->hairpin[27] = 6.4;
  rnapar->hairpin[28] = 6.4;
  rnapar->hairpin[29] = 6.5;
  rnapar->hairpin[30] = 6.5;

  /* internal loops asymmetry factor */

  rnapar->poppen[0] = 0.3;
  rnapar->poppen[1] = 0.3;
  rnapar->poppen[2] = 0.3;
  rnapar->poppen[3] = 0.3;


  /* Prohibit any  danglings and stackings to start with */
  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      for (k = 0; k < 4; k++)
	{ 
	 rnapar->dangle5[i][j][k] = BIGFLOAT;
	 rnapar->dangle3[i][j][k] = BIGFLOAT;

	 for (l = 0; l < 4; l++){
	   rnapar->stack[i][j][k][l]  = BIGFLOAT;
	   rnapar->tstckh[i][j][k][l] = BIGFLOAT;
	   rnapar->tstcki[i][j][k][l] = BIGFLOAT;}
	}

  /* introduce the allowed danglings and stackings */
  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      {
	rnapar->stack[0][3][i][j] = stack03[i][j];
	rnapar->stack[3][0][i][j] = stack30[i][j];
	rnapar->stack[1][2][i][j] = stack12[i][j];
	rnapar->stack[2][1][i][j] = stack21[i][j];
	rnapar->stack[2][3][i][j] = stack23[i][j];
	rnapar->stack[3][2][i][j] = stack32[i][j];

	rnapar->tstckh[0][3][i][j] = tstckh03[i][j];
	rnapar->tstckh[3][0][i][j] = tstckh30[i][j];
	rnapar->tstckh[1][2][i][j] = tstckh12[i][j];
	rnapar->tstckh[2][1][i][j] = tstckh21[i][j];
	rnapar->tstckh[2][3][i][j] = tstckh23[i][j];
	rnapar->tstckh[3][2][i][j] = tstckh32[i][j];

	rnapar->tstcki[0][3][i][j] = tstcki03[i][j];
	rnapar->tstcki[3][0][i][j] = tstcki30[i][j];
	rnapar->tstcki[1][2][i][j] = tstcki12[i][j];
	rnapar->tstcki[2][1][i][j] = tstcki21[i][j];
	rnapar->tstcki[2][3][i][j] = tstcki23[i][j];
	rnapar->tstcki[3][2][i][j] = tstcki32[i][j];

	rnapar->dangle5[0][i][j] = dangle50[i][j];
	rnapar->dangle5[1][i][j] = dangle51[i][j];
	rnapar->dangle5[2][i][j] = dangle52[i][j];
	rnapar->dangle5[3][i][j] = dangle53[i][j];

	rnapar->dangle3[0][i][j] = dangle30[i][j];
	rnapar->dangle3[1][i][j] = dangle31[i][j];
	rnapar->dangle3[2][i][j] = dangle32[i][j];
	rnapar->dangle3[3][i][j] = dangle33[i][j];
      }

  /* ultrastable tetrallops.
   * initialize all  tetraloop scores to zero 
   */
  for (tlp = 0; tlp < 256; tlp++)
    rnapar->tetraloop[tlp]  = 0.0;

  if ((iseq = (int **) malloc (13 * sizeof(int *))) == NULL)
    Die("malloc failed");

  for (ntlp = 0; ntlp < 13; ntlp++)
    if ((iseq[ntlp] = (int *) malloc (4 * sizeof(int))) == NULL)
      Die("malloc failed");

                 /* assign the sequences of the ultrastable tetraloops */
  for (ntlp = 0; ntlp < 13; ntlp++)
    for (i = 0; i < 4; i++)
      rnapar->tloop[ntlp][i] = tloop[ntlp][i];

                 /* assign the scores of the ultrastable tetraloops */
  for (ntlp = 0; ntlp < 13; ntlp++)
    {
      /* convert ultrastable sequences to 0..3 */
      IntizeSequence(rnapar->tloop[ntlp], 4, &iseq[ntlp]); 

      isc = iseq[ntlp][0] + iseq[ntlp][1]*4 + iseq[ntlp][2]*4*4 + iseq[ntlp][3]*4*4*4;		
				     
      rnapar->tetraloop[isc]  = -2.0;	
    }
}

/* Function: ParamIntSCFG()
 * 

 * Purpose:  Converts rna parameters at second order to
 *           integers, and changes sign.
 *     
 *           
 * Args:     rnapar - rna parameters
 *           cfg   - model (already allocated)                  
 */
int **
ParamIntSCFG(struct rnapar_2 *rnapar)
{
  int **icfg;
  int i, j, ip, jp;
  int size;
  int tlp;
  int asy; 

  icfg = AllocIntSCFG();

  /* Set everything flat to zero.
   */
  for (i = 0; i < NSTATES; i++)
    for (j = 0; j < NSTATES; j++)
      icfg[i][j] = 0;


  /* Use negative stacking energies as
   * scores for P-> transitions for 16 pair states
   */
  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)	 
      {
       if ((i+j == 3) || (i+j == 5)) /* determines base pairs (0,3),(3,0),(1,2),(2,1),(2,3),(3,2) */
	icfg[idxS][idxP(i,j)] = 1 * INTSCALE;   /* Watson-Crick */
       else
	icfg[idxS][idxP(i,j)] = -1 * BIGINT; /* PROHIBIT */
       
       for (ip = 0; ip < 4; ip++)
	 {
           icfg[idxL(ip)][idxP(i,j)] = -1 * (int)(rnapar->dangle5[ip][i][j] * INTSCALE);
	   icfg[idxR(ip)][idxP(i,j)] = -1 * (int)(rnapar->dangle3[ip][i][j] * INTSCALE);

	   for (jp = 0; jp < 4; jp++)
	     {
	       icfg[idxPS(i,j)][idxPS(ip,jp)]   = 
		                                -1 * (int)(rnapar->stack[i][j][ip][jp] * INTSCALE);
	       icfg[idxPL(i,j)][idxPL(ip,jp)] = 
                                                -1 * (int)(rnapar->tstckh[i][j][ip][jp] * INTSCALE);
	       icfg[idxPI(i,j)][idxPI(ip,jp)] = 
                                                -1 * (int)(rnapar->tstcki[i][j][ip][jp] * INTSCALE);
	     }
	 }
      }

  for (size = 0; size <= MAXRNALOOP; size++)
    {
      icfg[idxS][idxX(size)] =  -1 * (int)(rnapar->hairpin[size] * INTSCALE);
      icfg[idxS][idxV(size)] =  -1 * (int)(rnapar->bulge[size] * INTSCALE);
      icfg[idxS][idxW(size)] =  -1 * (int)(rnapar->inter[size] * INTSCALE);
    }
	
  for (asy = 0; asy < 4; asy++)
    icfg[idxS][idxWA(asy)] =  -1 * (int)(rnapar->poppen[asy] * INTSCALE);

  for (tlp = 0; tlp < 256; tlp++)
    icfg[idxS][idxTL(tlp)] =  -1 * (int)(rnapar->tetraloop[tlp] * INTSCALE);
  return icfg;
}





























