//                              -*- Mode: C++ -*- 
// DataWrap.h
// Copyright  1999 Laboratoire de Biologie Informatique et Thorique.
// Author           : Etienne Tourigny-Manning
// Created On       : Wed Dec  8 14:38:22 1999
// Last Modified By : 
// Last Modified On : 
// Update Count     : 0
// Status           : Unknown.
// 

#ifndef __DataWrap_h_
#define __DataWrap_h_

#include <iostream.h>
#include <vector.h>
#include <function.h>

//  Toute classe derivant de DataWrap peut redefinir void display(),
//  ostream &output et operator=

class DataWrap
{
protected:

  static unsigned int msSortIndex;
  vector< float > mCoord;
  
  // LIFECYCLE ------------------------------------------------------------

public:

  //on peut definir les constructeurs
  virtual ~DataWrap( void ) {}

  // OPERATORS ------------------------------------------------------------

  //pas sur que version const est utilisee
  float &operator[]( unsigned int index ); 
  const float &operator[]( unsigned int index ) const; 
  //rajouter check si = this
  inline virtual const DataWrap &operator=( const DataWrap &right )
    { mCoord = right.mCoord; return *this;}

  // METHODS --------------------------------------------------------------

  inline static unsigned int GetSortIndex( ) { return msSortIndex; }
  inline static unsigned int SetSortIndex( unsigned int index ) 
    { unsigned int old = msSortIndex; msSortIndex = index; return old; }
 
  inline unsigned int size() const { return mCoord.size(); } 
  inline virtual void display() { cout << *this; }
//    inline void clear() { 

  // I/O  -----------------------------------------------------------------

  friend ostream &operator<<( ostream &out, const DataWrap &theData );
  virtual ostream &output( ostream &out ) const;
}; 

ostream &operator<<( ostream &out, const DataWrap &theData );



//struct compare_dynamic->utilise index de chaque DataWrap a la place
//compare_static peut-il causer des conflits (peut-on trier deux vector a la fois?)

struct compare_static 
  : public binary_function < const DataWrap*, const DataWrap*, bool > {

  bool operator(  ) ( const DataWrap* d1, const DataWrap* d2 ) const {
    unsigned int index = DataWrap::GetSortIndex();
    if ( index >= d1->size() || index >= d2->size() ) {
      cerr << "compare_static(DataWrap): global index " 
	   << index << " too large... exiting" << endl;
      abort();
    }
//     cout<<"sorting by "<<index<<endl;
    return (*d1)[index] < (*d2)[index];
  }
};


template < int index > struct compareByIndex 
: public binary_function< const DataWrap*, const DataWrap*, bool > {

  bool operator() ( const DataWrap* d1, const DataWrap* d2 ) const {
    if ( index >= d1->size() || index >= d2->size() ) {
      cerr << "compareByIndex(DataWrap): index " << index << " too large..." << endl;
      abort();
    }
    return (*d1)[index] < (*d2)[index];
  }
};

typedef compareByIndex<0> compareAt0;


// on pourrait refaire avec < class Type, class Function >
// avec Function qui fait par defaut cout << *this
template < class Type > class DataOutput : public DataWrap
{
  Type mElement;

public:

  // LIFECYCLE ------------------------------------------------------------

  DataOutput( Type nElement ) : DataWrap( )
    { mElement = nElement; }
  DataOutput( Type nElement, float x ) : DataWrap( )
    { mElement = nElement; (*this)[0]=x; }
  DataOutput( Type nElement, float x, float y ) : DataWrap( )
    { mElement = nElement; (*this)[0]=x; (*this)[1]=y; }
  DataOutput( const DataOutput &copied ) : DataWrap( copied )
    { mElement = copied.mElement; }

  // OPERATORS ------------------------------------------------------------

  inline const DataOutput &operator=( const DataOutput &right ) 
    { DataWrap::operator=( right ); mElement=right.mElement; return *this;}

  // METHODS --------------------------------------------------------------

  inline virtual void display() { cout << *this; }

  // I/O  -----------------------------------------------------------------

  ostream &output( ostream &out ) const
    { DataWrap::output( out ); out << "Element: " << mElement << endl; return out; }

};


#endif


