//                              -*- Mode: C++ -*- 
// DensityPlotW.cc --- 
// 

#include "DensityPlotW.h"
#include "DataWrap.h"
#include <vector.h>

float DensityPlotW::msVar = 0.1;
float DensityPlotW::msCutoff = 0.00001;
 

// LIFECYCLE ------------------------------------------------------------

DensityPlotW::DensityPlotW( QWidget *parent=0, const char *name=0 ) 
  : PlotW( parent, name )  
{ 
  mRight = 15;
  mDataIndex = 0; 
  SetVar( ); 
  mFormatY = "%.2f";
  mConfig.grids = both;

  mVarSelect = new ValueSelectW( 0, "Variance", 0, 0.5, mVar );
  connect( mVarSelect, SIGNAL( ValueChanged( float ) ),
	   this, SLOT( ChangeVar( float ) ) );

  mAccel->connectItem( mAccel->insertItem( CTRL+Key_V ), 
		       this, SLOT( ShowVarSlider() ) );

  mPopup->insertSeparator( mPopupIndex++ );
  mPopup->insertItem( "&Variance", this, SLOT( ShowVarSlider() ), 
		      CTRL+Key_V, -1, mPopupIndex++ );

 }

 
// METHODS --------------------------------------------------------------

 
void DensityPlotW::UpdateScales( )
{
  if ( mDataCount == 0 ) {
    PlotW::UpdateScales( );
    return;
  }

  if ( ! mDirtyScreen ) return;

  unsigned int i, s, e, pix, center, width;
  int j;
  float tmp, ref;

  width = mWidth - mLeft - mRight;

  if (mScaleX.mVariable) {
    mScaleX.mMin = MAXFLOAT;
    mScaleX.mMax = -MAXFLOAT;
    for (s=0; s<mSerie.size(); s++) {
      Serie& serie = mSerie[s];
      for( e=0 ; e<serie.size() ; e++ ) {
	mScaleX.mMin = min( mScaleX.mMin, (*serie[e])[mDataIndex] - mVar*4 );
	mScaleX.mMax = max( mScaleX.mMax, (*serie[e])[mDataIndex] + mVar*4 );
      }
    } 
  }

  // Doit etre fait avant le calcul des mFloat puisque les min/max y seront utiliss
  // message de Etienne: ou sont-ils utilises???
  SetTick (mScaleX, 5);

  // assure que les axes entres par l'usager ne sont pas overrides
  if ( mScaleY.mVariable ) {
    mScaleY.mMin = 0;
    mScaleY.mMax = 0;
  }
  
  for (s=0; s<mSerie.size (); s++)  {
      Serie& serie = mSerie[s];
      if ( serie.size() == 0 ) break;
      
      // ce mecanisme permet de sorter selon un autre index que 0 mais
      // il faut faire TRES attention car ca utilise une variable
      // statique de DataWrap, msSortIndex(=0 par defaut)
      // On pourrait utiliser plutot une variable unique a chaque DataWrap
      // avec compare_dynamic p.e.
      uint old = DataWrap::SetSortIndex( mDataIndex );
      sort( serie.begin(), serie.end(), compare_static() );
      DataWrap::SetSortIndex( old );

      if ( serie.mFloat ) delete[] serie.mFloat;
      serie.mFloat = new float[ width ];

      center = 0;
      for (pix = 0; pix < width; ++pix) {
	ref = XPix2Val (mLeft + pix);
	serie.mFloat[pix] = 0;
	
	// On update le centre
	while ((*serie[center])[mDataIndex] < ref &&
	       center < serie.size () - 1)
	  ++center;

	// aller vers la gauche
	for (j = center; j >= 0; --j) {
	  tmp = Normal (XPix2Val (mLeft + pix), (*serie[j])[mDataIndex], mVar);
	  serie.mFloat[pix] += tmp / serie.size ();
    	  if (tmp < msCutoff && j != int (center)) break;
	}
	
	// aller vers la droite
	for (i = center + 1; i < serie.size (); ++i) {
	  tmp = Normal (XPix2Val (mLeft + pix), (*serie[i])[mDataIndex], mVar);
	  serie.mFloat[pix] += tmp / serie.size ();
  	  if (tmp < msCutoff) break;
	}

	// update de mScaleY.mMax ssi mScaleY est variable
	if ( mScaleY.mVariable ) 
	  mScaleY.mMax = max (mScaleY.mMax, serie.mFloat[pix]);
      }
    }

  // Doit etre fait apres puisque les min/max de Y sont calcul dans la
  // boucle precedante
  SetTick (mScaleY, 5);
}


void DensityPlotW::MyPaint( QPainter *painter, PaintConfig &config )
{  
  int pix;
  unsigned int s;

//    QPaintDeviceMetrics metric( painter->device() );

//     if ( !mSerie[k].mFloat || metric.width() != mWidth ) 
//    if ( (mDataCount > 0) && (metric.width() != mWidth) ) 
//      mScaleX.mDirty = mScaleY.mDirty = true;//      mDirtyNormal = true;

  PlotW::MyPaint( painter, config ); 

  for (s=0; s<mSerie.size(); s++) {
    Serie &serie = mSerie[s];
    if ( serie.size() == 0 ) break;
    if ( config.drawBlack ) painter->setPen( Qt::black );
    else painter->setPen( serie.mPen );
    painter->moveTo( mLeft, YVal2Pix( serie.mFloat[0] ) );
    for( pix=1 ; pix<mWidth-mLeft-mRight ; pix++ )
      painter->lineTo( mLeft+pix, YVal2Pix( serie.mFloat[pix] ) );
  }
}


void DensityPlotW::ShowVarSlider( )
{
  mVarSelect->show();  
}


void DensityPlotW::ChangeVar( float v ) 
{
//   cout<<"SLOT ChangeVar("<<v<<")"<<endl;
  SetVar(v); 
  mDirtyScreen = true; 
  Refresh(); 
}
 


void DensityPlotW::Pick( int x, int y )
{
  if ( mDataCount == 0 ) return;

  int closest_s=-1, closest_e = -1;
  float tmpDist, minDist = MAXFLOAT;
  unsigned int s, e;

  for( s=0 ; s<mSerie.size() ; s++ ){
    const Serie &serie = mSerie[s]; //const?
    for( e=0 ; e<serie.size() ; e++ ){
//        tmpDist = (int) fabs( x - XVal2Pix( (*serie[e])[mDataIndex] ) );
      tmpDist = fabs( XPix2Val( x ) - (*serie[e])[mDataIndex] );
      if ( tmpDist < minDist ){
	minDist = tmpDist;
	closest_s = s;
	closest_e = e;
      }
    }
  }
//    if ( (closest_s >= 0) && (minDist <= mPickingDist) ){
  if ( closest_s >= 0 ){
    mSerie[closest_s][closest_e]->display();  
  }
}


void DensityPlotW::Pick( int x1, int x2, int y1, int y2 )
{ 
  if ( mDataCount == 0 ) return;

  DataWrap* data;
  unsigned int s, e;
  float xMin = min( XPix2Val(x1), XPix2Val(x2) );
  float xMax = max( XPix2Val(x1), XPix2Val(x2) );

//    cout<<"Mouse picked region x=["<<xMin<<", "<<xMax<<"]"<<endl;

  for ( s=0; s<mSerie.size(); s++ ) {
    const Serie& serie = mSerie[s];
    for ( e=0; e<serie.size(); e++) {
      data = serie[e];
      if ( ((*data)[mDataIndex]>xMin) && ((*data)[mDataIndex]<xMax) )
	data->display();
    }
  }
  
}

void DensityPlotW::DisplayPick( int x1, int x2, int y1, int y2 )
{ 
  QPainter painter( this );
  painter.setPen( QColor(130, 0, 0) );

  bitBlt( this, 0, 0, &mOffScreen, 0, 0, width(), height() );

  painter.drawLine( x1, height(), x1, 0 );
  painter.drawLine( x2, height(), x2, 0 );
  
}
