//                              -*- Mode: C++ -*- 
// PlotW.cc --- 
// 

#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qwindowsstyle.h>
#include <qtextstream.h>

#include "DataWrap.h"

#include "PlotW.h"

// STATIC MEMBERS ------------------------------------------------------

int PlotW::msTimerInterval = 2000;             
QColor PlotW::msAxisColor = Qt::black;        //QColor( 0, 0, 0 );
QColor PlotW::msGridColor = Qt::lightGray;    //QColor( 180, 180, 180 );
QColor PlotW::msBackgroundColor = Qt::white;  //QColor( 255, 255, 255 );

// LIFECYCLE ------------------------------------------------------------

PlotW::PlotW( QWidget *parent, const char *name )
  : QWidget( parent, name )
{
  // configuration

  mAxisX = 0;
  mAxisY = 1;
  mPickingDist = 5;
  mLeft = 40;
  mRight = 15;
  mTop = 10;
  mBottom = 30;
  mFont = QFont( "helvetica", 11 );
  mFormatX = "%.1f";
  mFormatY = "%.1f";
  mTimerInterval = msTimerInterval;
  //timer desactive par defaut, si on veut l'utiliser il faudra voir
  mTimerActive = true;

  setBackgroundColor( msBackgroundColor ); 
  mConfig.axisColor = msAxisColor;
  mConfig.gridColor = msGridColor;
  mConfig.grids = none;
  mConfig.drawBlack = false;
  mPrintConfig.axisColor = Qt::black;
  mPrintConfig.gridColor = Qt::black;
  mPrintConfig.grids = none;
  mPrintConfig.drawBlack = false;

  // etat initial

  mDataCount = 0;
  mCurrentSerie=0;
  mWidth = mHeight = -1;
  mMouseX = mMouseY = -1;
  mLeftButton = false;
  mPopupOpen = false;
  mDirtyScreen = true;

  mScaleX.mMin = mScaleY.mMin = 0;
  mScaleX.mMax = mScaleY.mMax = 9.9;
  mScaleX.mVariable = mScaleY.mVariable = true; 
  mScaleX.mDirty = mScaleY.mDirty = true;


  // accels et menu
  // les accels ne sont pas "bindes" mais affiches dans la boite

  mAccel = new QAccel( this );
  mPopup = new QPopupMenu;

  mPopup->insertItem( "&Help", this, SLOT( Help() ), CTRL+Key_H );
  mAccel->connectItem( mAccel->insertItem( CTRL+Key_H ), this, SLOT( Help() ) );

  mPopup->insertItem( "&Save", this, SLOT( Save() ), CTRL+Key_S );
  mAccel->connectItem( mAccel->insertItem( CTRL+Key_S ), this, SLOT( Save() ) );

  mPopup->insertItem( "&Print", this, SLOT( Print() ), CTRL+Key_P );
  mAccel->connectItem( mAccel->insertItem( CTRL+Key_P ), this, SLOT( Print() ) );

  mPopup->insertItem( "&Options", this, SLOT( Options() ), CTRL+Key_O );
  mAccel->connectItem( mAccel->insertItem( CTRL+Key_O ), this, SLOT( Options() ) );

  mPopupIndex = mPopup->count();
  mPopup->insertSeparator( );

  mPopup->insertItem( "&Quit", qApp, SLOT( quit() ), CTRL+Key_Q );
  mAccel->connectItem( mAccel->insertItem( CTRL+Key_Q ), qApp, SLOT( quit() ) );

  mPopupBar = new QMenuBar();
  mPopupBar->insertItem("", mPopup);

  mTimer = new QTimer( this );
  connect( mTimer, SIGNAL( timeout() ), this, SLOT( Refresh() ) ); 
  if ( mTimerActive ) mTimer->start( mTimerInterval );

  SetupOptions( );

}


PlotW::~PlotW( void )
{
  delete mPopup;
  delete mPopupBar;
}


// OPERATORS ------------------------------------------------------------

// ACCESS ---------------------------------------------------------------

// METHODS --------------------------------------------------------------


void PlotW::SetupOptions( )
{
  mOptionDlg = new QTabDialog( this );
  mOptionDlg->setCaption( "Options" );
  mScaleWidget = new QWidget( this );

  QBoxLayout *layout;
  QButtonGroup *bg;
  QGroupBox *gbox;
  QGridLayout *grid;

  layout = new QBoxLayout( mScaleWidget, QBoxLayout::Down, 10 );

  // checkboxes
  bg = new QButtonGroup( 1, Qt::Horizontal, "Fixed Axes", mScaleWidget );
  layout->addWidget( bg );
  mScaler.cbx = new QCheckBox( "X Axis Fixed", bg );
  mScaler.cbx->setStyle( new QWindowsStyle() );
  mScaler.cbx->setChecked( ! mScaleX.mVariable );
  mScaler.cbx->setMinimumSize( mScaler.cbx->sizeHint() );
  mScaler.cby = new QCheckBox( "Y Axis Fixed", bg );
  mScaler.cby->setStyle( new QWindowsStyle() );
  mScaler.cby->setChecked( ! mScaleY.mVariable );
  mScaler.cby->setMinimumSize( mScaler.cby->sizeHint() );

  // X scale
  gbox = new QGroupBox( "X Scale", mScaleWidget );
  layout->addWidget( gbox );
  grid = new QGridLayout( gbox, 2, 2, 25, 20 );
  mScaler.lx1 = new QLabel( " ", gbox );
  mScaler.lx1->setAlignment( AlignCenter );
  mScaler.ex1 = new QLineEdit( gbox );
  mScaler.ex1->setAlignment( AlignCenter );
  mScaler.lx2 = new QLabel( " ", gbox );
  mScaler.lx2->setAlignment( AlignCenter );
  mScaler.ex2 = new QLineEdit( gbox );
  mScaler.ex2->setAlignment( AlignCenter );
  grid->addWidget( mScaler.lx1, 0, 0 );
  grid->addWidget( mScaler.ex1, 1, 0 );
  grid->addWidget( mScaler.lx2, 0, 1 );
  grid->addWidget( mScaler.ex2, 1, 1 );

  // Y scale
  gbox = new QGroupBox( "Y Scale", mScaleWidget );
  layout->addWidget( gbox );
  grid = new QGridLayout( gbox, 2, 2, 25, 20 );
  mScaler.ly1 = new QLabel( " ", gbox );
  mScaler.ly1->setAlignment( AlignCenter );
  mScaler.ey1 = new QLineEdit( gbox );
  mScaler.ey1->setAlignment( AlignCenter );
  mScaler.ly2 = new QLabel( " ", gbox );
  mScaler.ly2->setAlignment( AlignCenter );
  mScaler.ey2 = new QLineEdit( gbox );
  mScaler.ey2->setAlignment( AlignCenter );
  grid->addWidget( mScaler.ly1, 0, 0 );
  grid->addWidget( mScaler.ey1, 1, 0 );
  grid->addWidget( mScaler.ly2, 0, 1 );
  grid->addWidget( mScaler.ey2, 1, 1 );


  mOptionDlg->addTab( mScaleWidget, "Scales" );
  mOptionDlg->setApplyButton( );
  mOptionDlg->setCancelButton( );

  connect( mOptionDlg, SIGNAL( applyButtonPressed() ),
	   this, SLOT( ApplyOptions() ) );

}


//  Assure que mOptionDlg contient les bonnes valeurs,
//  effectue a l'ouverture du dialog et appui sur OK ou Apply
void PlotW::SetOptions( )
{
  QString str; 

  QTextOStream(&str) << "Min  =  " << mScaleX.mMin;
  mScaler.lx1->setText( str );
  
  str = "";
  QTextOStream(&str) << "Max  =  " << mScaleX.mMax;
  mScaler.lx2->setText( str );
  
  str = "";
  QTextOStream(&str) << "Min  =  " << mScaleY.mMin;
  mScaler.ly1->setText( str );
  
  str = "";
  QTextOStream(&str) << "Max  =  " << mScaleY.mMax;
  mScaler.ly2->setText( str );
  
}


//on pourrait faire cela plus efficacement avec Y=aX+b ou a et b sont des membres
//de Scale et ils sont updates avec mHeight et mWidth

int PlotW::XVal2Pix( float val ) const
{
  int ret;
  ret=(int)((val-mScaleX.mMin)/(mScaleX.mMax-mScaleX.mMin)*(mWidth-mLeft-mRight)+mLeft);
  return ret;
}

int PlotW::YVal2Pix( float val ) const
{
  float ret;
  ret=(mHeight-mBottom-(val-mScaleY.mMin)/(mScaleY.mMax-mScaleY.mMin)*(mHeight-mTop-mBottom));
  return (int)ceil(ret);
}


float PlotW::XPix2Val( int pix ) const
{
  float ret;
  ret=(float)(pix-mLeft)*(mScaleX.mMax-mScaleX.mMin)/(mWidth-mLeft-mRight) + mScaleX.mMin;
  return ret;
}


float PlotW::YPix2Val( int pix ) const
{
  float ret;
  ret=(float)(mHeight-pix-mBottom)*(mScaleY.mMax-mScaleY.mMin)/(mHeight-mTop-mBottom);
  return ret;
}

 
void PlotW::Reset( )
{
  mDataCount = 0;
  mCurrentSerie = 0;
  mSerie.clear ();
}


// Modifications: 
//  -double->float car cela introduisait des erreurs(scale.{mMin,mMax} en float)
//  -nb = ceil( range/incr ) et non (int)
//  -malheureusement, l'algo donne des valeurs "trop grandes" dans certains cas ou !scale.mVariable
//   dans ce cas, il faudrait   nb = (int)floor(range / incr) et rajouter des parametres aux Scale
//   et en tenir compte dans l'affichage des axes.
void PlotW::SetTick( Scale& scale, int goal )
{
  int l, start_i, nb;
  float range, incr, incr2, new_min, new_max;

  range = scale.mMax - scale.mMin;
  incr = range / goal;
  l = (int)floor( log10( incr ) );

  incr2 = incr * pow( 10, -l );
  if ( incr2 > 5 ) incr2 = 5;
  else if ( incr2 > 2 ) incr2 = 2;
  else incr2 = 1;

  incr = incr2 * pow( 10, l );

  start_i = (int)floor (scale.mMin/incr);
  nb = (int)ceil(range / incr) + 1;
  new_min = (float)(start_i * incr);
  new_max = new_min + (nb*incr);

  scale.mMin =  new_min;
  scale.mMax = new_max;
  scale.mIncr = incr;
  scale.mNb = nb;

}


int PlotW::AddSerie( vector< DataWrap* > *elements, QPen pen )
{
  Serie s( elements );
  s.mPen = pen;
  mSerie.push_back( s );

  mCurrentSerie = mSerie.size() - 1;
  mDataCount += elements->size();
  SetDirty();

  return mCurrentSerie;
}


int PlotW::AddSerie( QPen pen )
{
  Serie s;
  s.mPen = pen;
  mSerie.push_back( s );

  mCurrentSerie = mSerie.size() - 1;
//   cout<<"pen's red: "<<pen.color().red()<<endl;
//   cout<<"mCurrentSerie: "<<mCurrentSerie<<" size: "<<mSerie.size()<<endl;

  return mCurrentSerie;
}


int PlotW::AddData( DataWrap *data, unsigned int serieNo )
{
  if ( serieNo >= mSerie.size() ) {
    cerr << "PlotW::AddData, index " << serieNo
	 << " too big for mSerie's size: " << mSerie.size()
	 << endl;
    abort();
  }

  mCurrentSerie = serieNo;
  mSerie[mCurrentSerie].push_back(data);
  mDataCount++;
  SetDirty();
  
  return mCurrentSerie;
}


void PlotW::RemoveData( unsigned int serieNo, unsigned int dataNo )
{
  if ( serieNo < mSerie.size() && dataNo < mSerie[serieNo].size() ) {
    Serie::iterator s = mSerie[serieNo].begin();
    for ( unsigned int i=0; i<dataNo; i++, s++ );
    mSerie[serieNo].erase( s );
    mDataCount--;
    SetDirty();
  }
  else {
    cerr << "PlotW::RemoveData, item " << dataNo 
	 << " in serie " << serieNo 
	 << " does not exist!" << endl;
    abort();
  }    
}


void PlotW::SetDefaultColors( )
{
  mConfig.axisColor = msAxisColor;
  mConfig.gridColor = msGridColor;
  setBackgroundColor( msBackgroundColor ); 
}


//  Trouve les min et max de la Scale scale et cherchant les valeurs float 
//  a l'index index dans les Element de mSerie

void PlotW::UpdateScale( Scale& scale, int index )
{
  if ( ! scale.mDirty ) return;

  if (scale.mVariable) {
    if ( mDataCount == 0 ) {
      scale.mMin = 0;
      scale.mMax = 10;
    }
    else {
      float tmpFloat;
      unsigned int s,e;  //serie et element de serie
      
      float &Max = scale.mMax;
      float &Min = scale.mMin;
      Max = -MAXFLOAT;
      Min = MAXFLOAT;
      
      for( s=0; s<mSerie.size(); s++ ) { 
	Serie &serie = mSerie[s];
	for( e=0; e<serie.size() ; e++ ){
	  tmpFloat = (*serie[e])[index];//=(*(mSerie[ serie ].mX))[ i ];
	  Min = min( tmpFloat, Min );
	  Max = max( tmpFloat, Max );
	}
      }
    }
    
    // eviter des problemes lorsque mMin == mMax
    if ( scale.mMin == scale.mMax ) 
      scale.mMax+=10;
  }
  
  SetTick (scale, 7);

  mDirtyScreen = true;
  scale.mDirty = false;
}


//ne dessine que les axes et les echelles
void PlotW::MyPaint( QPainter *painter, PaintConfig &config ) 
{
  int height, width, originX, originY;
  int i, tmp;
  char tmpBuf[ 256 ];

  UpdateScales ();

  width = mWidth - mLeft - mRight;
  height = mHeight - mTop - mBottom;
  originX = mLeft;
  originY = mHeight - mBottom;
  
  painter->setClipRect (0, 0, mWidth, mHeight);

  painter->setFont( mFont );
  painter->setPen( config.axisColor );

  //graduations de l'axe X
  for( i=0 ; i<=mScaleX.mNb ; i++ ){
    tmp = (int)( originX+i*width/mScaleX.mNb );
    painter->drawLine( tmp, originY, tmp, originY+6 );
    if ( config.grids==x || config.grids==both ) {
      if (fabs(mScaleX.mIncr * i + mScaleX.mMin) < (mScaleX.mIncr/2)) {
	painter->setPen( config.gridColor );
	painter->drawLine( tmp, originY-1, tmp, originY-height );
	painter->setPen( config.axisColor );
      }
    }
    sprintf( tmpBuf, mFormatX, mScaleX.mIncr * i + mScaleX.mMin );
    painter->drawText( tmp, originY+mBottom/2, 0, 0,
		      DontClip | AlignCenter, tmpBuf );
  }


  for( i=0 ; i<=mScaleY.mNb ; i++ ){
    tmp = (int)( originY-i*height/mScaleY.mNb );
    painter->drawLine( originX, tmp, originX-6, tmp );
    if ( config.grids==y || config.grids==both ) {
      //      painter->setPen( config.gridColor );
      if (fabs(mScaleY.mIncr * i + mScaleY.mMin) < (mScaleY.mIncr/2)) {
	painter->setPen (QColor (0,0,200));
	painter->drawLine( originX, tmp, originX+width, tmp );
	painter->setPen( config.axisColor );
      }
    }
    sprintf( tmpBuf, mFormatY, mScaleY.mIncr * i + mScaleY.mMin );
    painter->drawText( originX-8, tmp, 0, 0,
		      DontClip | AlignVCenter | AlignRight, tmpBuf );
  }

  //les axes sont dessines apres les grids
  painter->drawLine( originX, originY, originX+width, originY );
  painter->drawLine( originX, originY, originX, originY-height );

  // On s'assure que les descendants ne peuvent pas dessiner par
  // dessus les axes
  painter->setClipRect (originX+1, originY - height, width - 1, height);
}


//  ne faire que la gestion des evenements de la souris
//  on laisse les calculs aux autres classes
void PlotW::mousePressEvent( QMouseEvent *mouse )
{
  if ( mouse->button() == RightButton ){
      mPopup->popup( mapToGlobal( mouse->pos() ), 1 );
  } 

  else if ( mouse->button() == LeftButton ) {
    mLeftButton = true;
    mMouseX = mouse->x();
    mMouseY = mouse->y();
//      cout<<"Mouse left pressed: ("<<mMouseX<<" ,"<<mMouseY<<")"<<endl;
  }

  else if ( mouse->button() == MidButton ) {
    mMouseX = mouse->x();
    mMouseY = mouse->y();
//      cout<<"Mouse left pressed: ("<<mMouseX<<" ,"<<mMouseY<<")"<<endl;
  }
}


void PlotW::mouseMoveEvent( QMouseEvent *mouse )
{
  if ( mLeftButton )
    DisplayPick( mouse->x(), mMouseX, mouse->y(), mMouseY );
}

void PlotW::mouseReleaseEvent( QMouseEvent *mouse )
{
  int x = mouse->x();
  int y = mouse->y();

  mLeftButton = false;

  if ( mouse->button() == LeftButton ) {
//      cout<<"Mouse left released: ("<<x<<" ,"<<y<<")"<<endl;
    
    if ( (x == mMouseX) && (y == mMouseY) )   
      Pick( x, y );
    else Pick( x, mMouseX, y, mMouseY );

    bitBlt( this, 0, 0, &mOffScreen, 0, 0, width(), height() );
  }

  else if ( mouse->button() == MidButton ) {
    cout << "Cursor on point ( " << XPix2Val( x ) 
	 << ", " << YPix2Val( y ) << " )" << endl;
  }

}


void PlotW::paintEvent( QPaintEvent* event )
{
  // n'est pas efficace lors d'une impression car on Paint apres l'impression
  if ( mDirtyScreen || mWidth != width() || mHeight != height() ) {
    QPaintDeviceMetrics metric( this );
    mWidth = metric.width();
    mHeight = metric.height();
    mDirtyScreen = true;
    Refresh( true );  
  }
  else 
    bitBlt( this, 0, 0, &mOffScreen, 0, 0, width(), height() );
}
   

void PlotW::Refresh( bool force ) 
{
  if ( !mDirtyScreen && !force ) return;

  mOffScreen.resize( width(), height() );
  mOffScreen.fill( backgroundColor() );  

  QPainter paint( &mOffScreen );
  MyPaint( &paint, mConfig );
  mDirtyScreen = false;

  bitBlt( this, 0, 0, &mOffScreen, 0, 0, width(), height() );
}


void PlotW::WritePpm( const char* filename )
{
  if (filename)
    mOffScreen.save (filename, "PPM");
  else {
    QString name = QFileDialog::getSaveFileName( 0, "*.ppm", this );
    if ( !name.isEmpty() ) {
      mOffScreen.save (name, "PPM");
    }
  }
}


void PlotW::Print( )
{ 
  Refresh( );

  QPrinter printer;  
  printer.setOutputToFile( true );
  printer.setPageSize( QPrinter::Letter );

  if ( printer.setup( this ) ) {
    int width, height;//, w, h;
    QPaintDeviceMetrics metric( &printer );
    width = metric.width();
    height = metric.height();

    // rajuster les dimensions du graphique sur la page 
    // en gardant les memes proportions que le graphique visuel
    if ( mWidth > width || mHeight > height ) {
      height = width*mHeight/mWidth;
      if ( height > metric.height() ) {
	height = metric.height();
	width = height*mWidth/mHeight;
      }
      mWidth = width;
      mHeight = height;
      mDirtyScreen = true;
    }

    // backup, paint puis restore
//      mDrawPS = true;
//      QColor axisColor = mAxisColor;
//      mAxisColor = Qt::black;

    QPainter paint( &printer );
    MyPaint( &paint, mPrintConfig );

//      mAxisColor = axisColor;
//      mDrawPS = false;
    mDirtyScreen = false;
  }
}


void PlotW::Options( ) 
{
  if ( !mOptionDlg->isVisible() ) SetOptions();
  mOptionDlg->show(); 
}  
	   

// Applique les options choisies 
// ne verifie pas la validite des intervalles pour les axes
// n'est pas optimal pour les Refresh quand il n'y a rien a changer
// Comportement etrange en Y pour DensityPlotW
// pas teste lorsque le programmeur fait Set{X,Y}Scale avant de lancer le PlotW
void PlotW::ApplyOptions( )
{
  bool ok = false, changeScales = true;
  float x1=0, x2=0, y1=0, y2=0;

  // check input
  if ( mScaler.cbx->isChecked() ) {
    x1 = mScaler.ex1->text().toFloat( &ok );
    if (!ok) changeScales = false;
    x2 = mScaler.ex2->text().toFloat( &ok );
    if (!ok) changeScales = false;
    if ( x1 >= x2 ) changeScales = false;
  }

  if ( mScaler.cby->isChecked() ) {
    y1 = mScaler.ey1->text().toFloat( &ok );
    if (!ok) changeScales = false;
    y2 = mScaler.ey2->text().toFloat( &ok );
    if (!ok) changeScales = false;
    if ( y1 >= y2 ) changeScales = false;
  }

  if ( changeScales ) {
    mScaleX.mVariable = !mScaler.cbx->isChecked();
    mScaleY.mVariable = !mScaler.cby->isChecked();
    
    if ( ! mScaleX.mVariable ) {
      mScaleX.mMin = x1;
      mScaleX.mMax = x2;
    }
    if ( ! mScaleY.mVariable ) {
      mScaleY.mMin = y1;
      mScaleY.mMax = y2;
    }
    
    SetDirty( );
    Refresh( );
    SetOptions( );
  }

}







