//                              -*- Mode: C++ -*- 
// PlotW.h
// Copyright  1999 Laboratoire de Biologie Informatique et Thorique.
// Author           : Etienne Tourigny-Manning
// Created On       : Wed Dec  8 14:39:05 1999
// Last Modified By : 
// Last Modified On : 
// Update Count     : 0
// Status           : Unknown.
// 

#ifndef __PlotW_h_
#define __PlotW_h_

#include <iostream.h>
#include <stdio.h>
#include <values.h>
#include <vector.h>
#include <math.h>

#include <qwidget.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qtooltip.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qpixmap.h>
#include <qfiledialog.h>
#include <qjpegio.h>
#include <qprinter.h>
#include <qmenubar.h>
#include <qtimer.h>
#include <qtabdialog.h>

class QCheckBox;

class DataWrap;

class PlotW  : public QWidget
{
  Q_OBJECT

protected:

  class Serie : public vector< DataWrap* > {

  public:
    QPen mPen;
    int mIndex;
    int *mInt;
    float *mFloat;  //Pour courbe Density ou autre
    
    Serie( ) : vector< DataWrap* > () { mIndex = 0; mInt = 0; mFloat = 0; }
    Serie( vector< DataWrap* > *vect ) 
      : vector< DataWrap* > ( *vect ) { mIndex = 0; mInt = 0; mFloat = 0; }
    Serie( const Serie& copied ) : vector< DataWrap* > (copied)
      { mPen = copied.mPen; mIndex = copied.mIndex; mInt = 0; mFloat = 0; } 
    virtual ~Serie( ) { delete mInt; delete mFloat; } 
//     const Serie &operator=( const Serie &right ) : 
  };

  struct Scale {
    float mMin;
    float mMax;
    float mIncr;
    float mNb;
    int mBorder;
    bool mVariable;
    bool mDirty;
  };

  struct PlotScaler {
    QLabel *lx1, *lx2, *ly1, *ly2 ;
    QLineEdit *ex1, *ex2, *ey1, *ey2;
    QCheckBox *cbx, *cby;
    float x1, x2, y1, y2;
  };

  enum Grids { none, x, y, both };

  struct PaintConfig {
//      int width;         //mWidth
//      int height;        //mHeight
    QColor axisColor;  //mAxisColor
    QColor gridColor;  //mGridColor
    Grids grids;       //mShowGrids
    bool drawBlack;    //dessine les elements en noir, pour PS
  };

  // variables fondamentales
  vector< Serie > mSerie;
  Scale mScaleX, mScaleY;
  QPixmap mOffScreen;
  QMenuBar *mPopupBar;
  QPopupMenu *mPopup;
  QAccel *mAccel;
  QTimer *mTimer;
  QTabDialog *mOptionDlg;
  QWidget *mScaleWidget;
  PlotScaler mScaler;

  // variables correspondant a l'etat actuel
  int mDataCount;
  int mCurrentSerie;
  int mWidth, mHeight;        //dimensions du graphique (<= aux dimensions du QPainDevice)
  int mMouseX, mMouseY;       //utile aux mouseEvent
  bool mLeftButton;
  int mPopupIndex;
  bool mPopupOpen;
  bool mDirtyScreen;          //on doit redessiner le widget
//    bool mShowGrids;
//    bool mDrawPS;               //on dessine en ps, donc noir sur fond blanc
                              //ce mecanisme est a revoir car trop hard-code
  
 // variables de configuration
  int mAxisX, mAxisY;                //index dans les DataWrap correspondant aux axes
  int mPickingDist;                  //distance de picking
  int mLeft, mRight, mTop, mBottom;  //bordures
//    QColor mAxisColor, mGridColor;
  QFont mFont;
  char *mFormatX;
  char *mFormatY;
  int mTimerInterval;
  bool mTimerActive;
  PaintConfig mConfig;
  PaintConfig mPrintConfig; 

  // variables statiques utilisees comme valeur par defaut
  static int msTimerInterval;
  static QColor msAxisColor, msGridColor, msBackgroundColor;


public:

  // LIFECYCLE ------------------------------------------------------------

  PlotW( QWidget *parent=0, const char *name=0 );
  virtual ~PlotW( void );

  // METHODS --------------------------------------------------------------

public:

  int AddSerie( vector< DataWrap* > *elements, QPen pen );
  int AddSerie( QPen pen );
  int AddData( DataWrap *data, unsigned int serieNo );
  int AddData( DataWrap *data ) { return AddData( data, mCurrentSerie ); }
  void RemoveData( unsigned int serieNo, unsigned int dataNo );

  virtual void Reset( );
  void SetXScale( float min, float max ) { SetScale( mScaleX, min, max ); }
  void SetYScale( float min, float max ) { SetScale( mScaleY, min, max ); }
  void SetPickingDist( int d ) { mPickingDist = (int)fabs(d); }
  void SetDirty( ) { mDirtyScreen = mScaleX.mDirty = mScaleY.mDirty = true; }  

protected:
  
  void SetupOptions( );
  void SetOptions( );

  float XPix2Val( int pix ) const;
  int XVal2Pix( float val ) const;
  float YPix2Val( int pix ) const; 
  int YVal2Pix( float val ) const;

  void SetDefaultColors( );

  virtual void UpdateScales( )
    { UpdateScale( mScaleX, mAxisX ); UpdateScale( mScaleY, mAxisY ); }
  void UpdateScale( Scale& scale, int index );
  void SetTick( Scale& scale, int goal );
  void SetScale( Scale& scale, float min, float max )
    { scale.mMin=min; scale.mMax=max; scale.mVariable = false; }

  void paintEvent( QPaintEvent *event );
  virtual void MyPaint( QPainter *painter, PaintConfig &config );

  void mousePressEvent( QMouseEvent *e );
  void mouseMoveEvent( QMouseEvent *e );
  void mouseReleaseEvent( QMouseEvent *e );

  virtual void Pick( int x, int y ) {}
  virtual void Pick( int x1, int x2, int y1, int y2 ) {}
  virtual void DisplayPick( int x1, int x2, int y1, int y2 ) {}

  void WritePpm ( const char* filename = NULL );


  // Signals --------------------------------------------------------------
  
  // Slots ----------------------------------------------------------------
  
public slots:

  void Refresh( ) { Refresh( mDirtyScreen ); }
  void Refresh( bool force );
  void SetBackgroundColor( const QColor &color ) 
    { setBackgroundColor( QColor(color) ); Refresh( true ); }

  void Help( ) { cout << "Help slot selected" << endl; }
  void Save( ) { Refresh(); WritePpm( ); }
  void Print( );
  void Options( );
  void ApplyOptions( );
  
//  //  void repaint( void ){ QWidget::repaint( false ); }
};

#endif
