//                              -*- Mode: C++ -*- 
// ScatterPlotW.cc --- 
// 

#include "ScatterPlotW.h"
#include "DataWrap.h"


// LIFECYCLE ------------------------------------------------------------

ScatterPlotW::ScatterPlotW( QWidget *parent, const char *name )
  : PlotW( parent, name )
{
  //  msAxisColor = Qt::lightGray;        //QColor( 180, 180, 180 );
  //  msGridColor = QColor( 90, 90, 90 );
  //  msBackgroundColor = QColor( 0, 0, 50 );
  SetDefaultColors();
//    mShowGrids = true;
  mConfig.grids = both;
  mPrintConfig.drawBlack = true;
}

// OPERATORS ------------------------------------------------------------

// ACCESS ---------------------------------------------------------------

// METHODS --------------------------------------------------------------

void ScatterPlotW::MyPaint( QPainter *painter, PaintConfig &config ) 
{
  PlotW::MyPaint(painter, config); 

  if ( mDataCount == 0 ) return;

  int x, y;  //pixels
  unsigned int s, e;  //serie et element de serie
 
  for( s=0 ; s<mSerie.size() ; s++ ){
    Serie &serie = mSerie[s];
    if ( serie.size() == 0 ) break;
    if ( config.drawBlack ) painter->setPen( Qt::black );
    else painter->setPen( serie.mPen );
    for( e=0; e<serie.size() ; e++ ){
      x = XVal2Pix((*serie[e])[mAxisX]);
      y = YVal2Pix((*serie[e])[mAxisY]);
      painter->drawEllipse( x-2, y-2, 4, 4 );
      //painter->drawLine(x-2,y,x+2,y);
      //painter->drawLine(x,y-2,x,y+2);
      // Line Plot:
      if (e!=0)
	painter->drawLine(XVal2Pix((*serie[e-1])[mAxisX]), YVal2Pix((*serie[e-1])[mAxisY]), x, y);
    }
  }
}


//  Revoir cet algorithme pour des points a meme distance
//  de l'endroit clique
void ScatterPlotW::Pick( int x, int y )
{
  if ( mDataCount == 0 ) return;

  float dx, dy;
  int closest_s=-1, closest_e = -1;
  float tmpDist, minDist = MAXFLOAT;
  unsigned int s, e;

  //      cout<<"Mouse picked x="<<x<<" y="<<y<<endl;
  
  for( s=0 ; s<mSerie.size() ; s++ ){
    const Serie &serie = mSerie[s]; //const?
    for( e=0 ; e<serie.size() ; e++ ){
      dx = x - XVal2Pix( (*serie[e])[mAxisX] );
      dy = y - YVal2Pix( (*serie[e])[mAxisY] );
      tmpDist = sqrt( dx*dx + dy*dy );
      if ( tmpDist < minDist ){
	minDist = tmpDist;
	closest_s = s;
	closest_e = e;
      }
    }
  }
  if ( (closest_s >= 0) && (minDist <= mPickingDist) ){
    //cout<<*mSerie[closest_s].mElement[closest_e];
    //cout<<closest_s;
    mSerie[closest_s][closest_e]->display();  
  }
}


void ScatterPlotW::Pick( int x1, int x2, int y1, int y2 )
{ 
  if ( mDataCount == 0 ) return;

  DataWrap* data;
  unsigned int s, e;
  float xMin = min( XPix2Val(x1), XPix2Val(x2) );
  float xMax = max( XPix2Val(x1), XPix2Val(x2) );
  float yMin = min( YPix2Val(y1), YPix2Val(y2) );
  float yMax = max( YPix2Val(y1), YPix2Val(y2) );

//    cout<<"Mouse picked region x=["<<xMin<<", "<<xMax<<"] y=["<<yMin<<", "<<yMax<<"]"<<endl;

  for ( s=0; s<mSerie.size(); s++ ) {
    const Serie& serie = mSerie[s];
    for ( e=0; e<serie.size(); e++) {
      data = serie[e];
      if ( ((*data)[mAxisX]>xMin) && ((*data)[mAxisX]<xMax) && ((*data)[mAxisY]>yMin) && ((*data)[mAxisY]<yMax) )
	data->display();
    }
  }
  
}




void ScatterPlotW::DisplayPick( int x1, int x2, int y1, int y2 )
{ 
  QPainter painter( this );
  painter.setPen( QColor(130, 0, 0) );

  bitBlt( this, 0, 0, &mOffScreen, 0, 0, width(), height() );

  painter.drawRect( x1, y1, (x2-x1), (y2-y1) );

}
